//======================================================================
//          IrisMacro.h
//            IRIS W}N֐
//
//          Copyright (C) 2002-2003 NINTENDO Co.,Ltd.
//======================================================================
#ifndef _IRIS_MACRO_H
#define _IRIS_MACRO_H

#ifdef __cplusplus
extern "C" {
#endif

#include <IrisTarget.h>
#include <IrisTypes.h>
#include <IrisDefine.h>
#include <IrisMemoryMap.h>
#include <IrisSystemCall.h>
#include <IrisUTL.h>
#include <IrisOS.h>


//----------------------------------------------------------------------
//                          J[
//----------------------------------------------------------------------

#define RGB555(r, g, b)                                             \
                                                                    \
        ((r) << RED5_SHIFT | (g) << GREEN5_SHIFT | (b) << BLUE5_SHIFT)

#define RGBA5551(r, g, b, a)                                        \
                                                                    \
        ((r) << RED5_SHIFT | (g) << GREEN5_SHIFT | (b) << BLUE5_SHIFT | (a) << ALPHA1_SHIFT)

#define RGBF5551(r, g, b, flag)                                     \
                                                                    \
        ((r) << RED5_SHIFT | (g) << GREEN5_SHIFT | (b) << BLUE5_SHIFT | (flag) << 15)


#define RGB888(r, g, b)                                             \
                                                                    \
        ((r) << RED8_SHIFT | (g) << GREEN8_SHIFT | (b) << BLUE8_SHIFT)

#define RGBA8888(r, g, b, a)                                        \
                                                                    \
        ((r) << RED8_SHIFT | (g) << GREEN8_SHIFT | (b) << BLUE8_SHIFT | (a) << ALPHA8_SHIFT)


//----------------------------------------------------------------------
//                          W
//----------------------------------------------------------------------

#define XYZ101010(x, y, z)        (((x) & X10_MASK) << X10_SHIFT    \
                                 | ((y) & X10_MASK) << Y10_SHIFT    \
                                 | ((z) & X10_MASK) << Z10_SHIFT)

#define XY1616(x, y)              (((x) & X16_MASK) | (y) << Y16_SHIFT)
#define Z16(z)                     ((z) & X16_MASK)

#define ZW1616(z, w)              (((z) & X16_MASK) | (w) << Y16_SHIFT)
#define HD1616(h, d)              (((h) & X16_MASK) | (d) << Y16_SHIFT)

#define ST1616(s, t)              (((s) & TEX_S_MASK  ) | (t) << TEX_T_SHIFT)


//----------------------------------------------------------------------
//                          Cg
//----------------------------------------------------------------------

#define LIGHT_VECTOR(no, x, y, z) ((no)<<LIGHT_ID_SHIFT | XYZ101010((x), (y), (z)))
#define LIGHT_COLOR( no, r, g, b) ((no)<<LIGHT_ID_SHIFT | RGB555(   (r), (g), (b)))


//----------------------------------------------------------------------
//              eNX`C[Wpbg AhXϊ
//----------------------------------------------------------------------

// X^eBbNXgp
#define ADDR_TO_TEXIMAGE_BASE(basep)        (((basep) >> TEX_IMAGE_BASE_SHIFT) & TEX_IMAGE_BASE_MASK)
#define ADDR_TO_TEXPLTT_BASE(basep, mode)   (((basep) >> mode##_SHIFT) & TEX_PLTT_BASE_MASK)

// A^Cp
#define Addr2TexImageBase(basep)            (((u32 )(basep) >> TEX_IMAGE_BASE_SHIFT) & TEX_IMAGE_BASE_MASK)
#define Addr2TexPlttBase(basep, mode)       ( (u32 )(basep) >> "\0\4\3\4\4\4\4"[mode])


//----------------------------------------------------------------------
//                          Z
//----------------------------------------------------------------------

// p[^ݒiʏŁj

#define SetDiv(numer, denom, numBits, denBits)						\
{																	\
	*(vu32 *)REG_DIVCNT = DIV_##numBits##_##denBits##BIT_MODE;		\
    SetDivImm(numer, denom, numBits, denBits);						\
}

// p[^ݒi[hݒȗŁj

#define SetDivImm(numer, denom, numBits, denBits)					\
{																	\
    *(vs##numBits *)REG_DIV_NUMER = (numer);						\
    *(vs##denBits *)REG_DIV_DENOM = (denom);						\
}

// ZI҂

#define IsDivBusy()					(((vu8 *)REG_DIVCNT)[1] & (DIV_BUSY>>8))
#define WaitDiv()													\
{																	\
	while (IsDivBusy())	;											\
}

// ʓǂݏoiZImFŁj

#define DivResultImm(bits)			(*(vs##bits *)REG_DIV_RESULT)
#define DivRemResultImm(bits)		(*(vs##bits *)REG_DIVREM_RESULT)

// ʓǂݏoiʏŁj

#define DivResult(bits)				DivResult##bits##()
#define DivRemResult(bits)			DivRemResult##bits##()

__inline static s64 DivResult64(void)
{
	WaitDiv();

	return  DivResultImm(64);
}
__inline static s64 DivRemResult64(void)
{
	WaitDiv();

	return  DivRemResultImm(64);
}

__inline static s32 DivResult32(void)
{
	WaitDiv();

	return  DivResultImm(32);
}
__inline static s32 DivRemResult32(void)
{
	WaitDiv();

	return  DivRemResultImm(32);
}

__inline static s16 DivResult16(void)
{
	WaitDiv();

	return  DivResultImm(16);
}
__inline static s16 DivRemResult16(void)
{
	WaitDiv();

	return  DivRemResultImm(16);
}

__inline static s8 DivResult8(void)
{
	WaitDiv();

	return  DivResultImm(8);
}
__inline static s8 DivRemResult8(void)
{
	WaitDiv();

	return  DivRemResultImm(8);
}

//----------------------------------------------------------------------
//                      Z
//----------------------------------------------------------------------

// p[^ݒiʏŁj

#define SetSqrt(param, bits)										\
{																	\
	*(vu32 *)REG_SQRTCNT = SQRT_##bits##BIT_MODE;					\
    SetSqrtImm(param, bits);										\
}

// p[^ݒi[hݒȗŁj

#define SetSqrtImm(param, bits)										\
{																	\
    *(vu##bits *)REG_SQRT_PARAM = (param);							\
}

// ZI҂

#define IsSqrtBusy()				(((vu8 *)REG_SQRTCNT)[1] & (SQRT_BUSY>>8))
#define WaitSqrt()													\
{																	\
	while (IsSqrtBusy())   ;										\
}

// ʓǂݏoiZImFŁj

#define SqrtResultImm(bits)				(*(vu##bits *)REG_SQRT_RESULT)

// ʓǂݏoiʏŁj

#define SqrtResult(bits)				SqrtResult##bits##()

__inline static u32 SqrtResult32(void)
{
	WaitSqrt();

	return  SqrtResultImm(32);
}
__inline static u16 SqrtResult16(void)
{
	WaitSqrt();

	return  SqrtResultImm(16);
}
__inline static u8 SqrtResult8(void)
{
	WaitSqrt();

	return  SqrtResultImm(8);
}


//----------------------------------------------------------------------
//                      f[^ǂݍ
//----------------------------------------------------------------------

#define ReadData(readp, bit)    (*(vu##bit *)(readp))

//----------------------------------------------------------------------
//                      f[^
//----------------------------------------------------------------------

#define WriteData(writep, data, bit)                    \
                                                        \
    (*(vu##bit *)(writep) = (data))

//----------------------------------------------------------------------
//                      f[^ Rs[
//----------------------------------------------------------------------

#define CopyData(readp, writep, bit)                    \
                                                        \
    (*(vu##bit *)(writep) = *(vu##bit *)(readp))


//----------------------------------------------------------------------
//                  AhXEobt@̗L
//----------------------------------------------------------------------

#ifdef  __CC_ARM
#define EnableAlignBuf(alignBuf)                        \
{                                                       \
    vu16 alignBufTmp = *(vu16 *)alignBuf;               \
}
#else
#define EnableAlignBuf(alignBuf) 	((void) 0)
#endif

//----------------------------------------------------------------------
//                  荞݃AhX Zbg
//----------------------------------------------------------------------

#define SetIntrAddr(intrp)                              \
                                                        \
    (*(vu32 *)INTR_VECTOR_BUF = (vu32 )intrp)

//E荞ݏ̃AhXZbg܂B


//----------------------------------------------------------------------
//                      cl` Zbg
//----------------------------------------------------------------------

//#define SIMULATOR
#ifndef SIMULATOR
#define DmaSet(dmaNo, srcp, destp, dmaCntData)                       \
{                                                                    \
    vu32 *dmaCntp = &((vu32 *)REG_DMA0)[dmaNo * 3];                  \
    dmaCntp[0] = (vu32 )(srcp);                                      \
    dmaCntp[1] = (vu32 )(destp);                                     \
    dmaCntp[2] = (vu32 )(dmaCntData);                                \
    {u32 dummy = dmaCntp[2];}                                        \
    {u32 dummy = dmaCntp[2];}                                        \
}
#define DmaSetAsync(dmaNo, srcp, destp, dmaCntData)                  \
{                                                                    \
    vu32 *dmaCntp = &((vu32 *)REG_DMA0)[dmaNo * 3];                  \
    dmaCntp[0] = (vu32 )(srcp);                                      \
    dmaCntp[1] = (vu32 )(destp);                                     \
    dmaCntp[2] = (vu32 )(dmaCntData);                                \
}
#else
#define DmaSet(dmaNo, srcp, destp, dmaCntData)                       \
{                                                                    \
    int    i;                                                        \
    for (i=0; i<(dmaCntData & 0x1ffff); i++)                         \
        if ((dmaCntData) & DMA_SRC_FIX) {                            \
            if ((dmaCntData) & DMA_32BIT_BUS)                        \
                    ((vu32 *)(destp))[i] = ((vu32 *)(srcp))[0];      \
            else    ((vu16 *)(destp))[i] = ((vu16 *)(srcp))[0];      \
         } else {                                                    \
            if ((dmaCntData) & DMA_32BIT_BUS)                        \
                ((vu32 *)(destp))[i] = ((vu32 *)(srcp))[i];          \
            else    ((vu16 *)(destp))[i] = ((vu16 *)(srcp))[i];      \
        }                                                            \
}
#define DmaSetAsync(dmaNo, srcp, destp, dmaCntData)                  \
        DmaSet(     dmaNo, srcp, destp, dmaCntData)
#endif

//EDMARg[Ƀp[^Zbg܂B
//ESIMULATER`CPUŃV~[g܂B
//  GDBŃfobOꍇȂǂɗLłB
//EŌDMAN҂̂߂"LDR"߂}܂B
//
//EF
//  dmaNo       DMAԍ
//  srcp        \[XAhX
//  destp       fXeBl[VAhX
//  dmaCntData  p[^f[^
//
//CPURAM̃vOɂDMAN
//  ̖̎߂Ɏs܂B
//  āADMAɓ]CPUɂĕύX悤ƂꍇA
//  ǂݍ݁^߂̊ԂDMANĂ܂܂̂ŁA
//  DMA̓]̃f[^Ӑ}ĂȂlɂȂꍇ܂B
//  ̏ꍇɂWaitDma()𒼌ɑ}āADMAI
//  ǂ`FbNƑR[hւ̉emɉł܂B


//----------------------------------------------------------------------
//                      cl` NA
//----------------------------------------------------------------------

#define DmaClear(dmaNo, data, destp, size, bit)             \
{                                                           \
    vu32 *srcp = &((vu32 *)REG_DMA0_CLR_DATA)[dmaNo];       \
    *srcp = (data);                                         \
    DmaSet(dmaNo, srcp, destp, (                            \
        DMA_ENABLE         | DMA_TIMMING_IMM |              \
        DMA_SRC_FIX        | DMA_DEST_INC    |              \
        DMA_##bit##BIT_BUS | ((size)/(bit/8))));            \
}

#define DmaClearIf(dmaNo, data, destp, size, bit)           \
{                                                           \
    vu32 *srcp = &((vu32 *)REG_DMA0_CLR_DATA)[dmaNo];       \
    *srcp = (data);                                         \
    DmaSet(dmaNo, srcp, destp, (                            \
        DMA_ENABLE         | DMA_TIMMING_IMM |              \
        DMA_IF_ENABLE      |                                \
        DMA_SRC_FIX        | DMA_DEST_INC    |              \
        DMA_##bit##BIT_BUS | ((size)/(bit/8))));            \
}

#define DmaClearArray(  dmaNo, data, destp, bit)            \
        DmaClear(       dmaNo, data, destp, sizeof(destp), bit)

#define DmaClearArrayIf(dmaNo, data, destp, bit)            \
        DmaClearIf(     dmaNo, data, destp, sizeof(destp), bit)

//EDMARAMNA܂B
//ENAf[^̓X^bNɒuAfXeBl[VփRs[܂B
//EDmaClearIf^DmaClearArrayIf͏IɊ荞ݗv𔭐܂B
//EDmaClearArray^DmaClearArrayIf̓fXeBl[VzŜNA܂B
//
//EF
//  dmaNo       DMAԍ
//  data        NAf[^
//  destp       fXeBl[VAhX
//  size        NAoCg
//  bit         ]rbgi16|32j
//
//
//CPURAM̃vOɂDMAN
//  ̖̎߂Ɏs܂B
//  āADMAɓ]CPUɂĕύX悤ƂꍇA
//  ǂݍ݁^߂̊ԂDMANĂ܂܂̂ŁA
//  DMA̓]̃f[^Ӑ}ĂȂlɂȂꍇ܂B
//  ̏ꍇɂWaitDma()𒼌ɑ}āADMAI
//  ǂ`FbNƑR[hւ̉emɉł܂B

//----------------------------------------------------------------------
//                      cl` Rs[
//----------------------------------------------------------------------

#define DmaCopy(dmaNo, srcp, destp, size, bit)              \
                                                            \
    DmaSet(dmaNo, srcp, destp,  (                           \
        DMA_ENABLE         | DMA_TIMMING_IMM |              \
        DMA_SRC_INC        | DMA_DEST_INC    |              \
        DMA_##bit##BIT_BUS | ((size)/((bit)/8))))

#define DmaCopyIf(dmaNo, srcp, destp, size, bit)            \
                                                            \
    DmaSet(dmaNo, srcp, destp,  (                           \
        DMA_ENABLE         | DMA_TIMMING_IMM |              \
        DMA_IF_ENABLE      |                                \
        DMA_SRC_INC        | DMA_DEST_INC    |              \
        DMA_##bit##BIT_BUS | ((size)/(bit/8))))

#define DmaCopyArray(  dmaNo, srcp, destp, bit)             \
        DmaCopy(       dmaNo, srcp, destp, sizeof(srcp), bit)

#define DmaCopyArrayIf(dmaNo, srcp, destp, bit)             \
        DmaCopyIf(     dmaNo, srcp, destp, sizeof(srcp), bit)

//EDMAŃRs[܂B
//EDmaCopyIf^DmaCopyArrayIf͏IɊ荞ݗv𔭐܂B
//EDmaCopyArray^DmaCopyArrayIf̓\[XzŜRs[܂B
//
//EF
//  dmaNo       DMAԍ
//  srcp        \[XAhX
//  destp       fXeBl[VAhX
//  size        ]oCg
//  bit         ]rbgi16|32j
//
//
//CPURAM̃vOɂDMAN
//  ̖̎߂Ɏs܂B
//  āADMAɓ]CPUɂĕύX悤ƂꍇA
//  ǂݍ݁^߂̊ԂDMANĂ܂܂̂ŁA
//  DMA̓]̃f[^Ӑ}ĂȂlɂȂꍇ܂B
//  ̏ꍇɂWaitDma()𒼌ɑ}āADMAI
//  ǂ`FbNƑR[hւ̉emɉł܂B

//----------------------------------------------------------------------
//                  guNcl` Rs[
//----------------------------------------------------------------------

#define H_DmaCopy(dmaNo, srcp, destp, size, bit)            \
                                                            \
    DmaSet(dmaNo, srcp, destp,  (                           \
        DMA_ENABLE         | DMA_TIMMING_H_BLANK |          \
        DMA_SRC_INC        | DMA_DEST_RELOAD     |          \
        DMA_CONTINUOUS_ON  |                                \
        DMA_##bit##BIT_BUS | ((size)/((bit)/8))))

#define H_DmaCopyIf(dmaNo, srcp, destp, size, bit)          \
                                                            \
    DmaSet(dmaNo, srcp, destp,  (                           \
        DMA_ENABLE         | DMA_TIMMING_H_BLANK |          \
        DMA_IF_ENABLE      |                                \
        DMA_SRC_INC        | DMA_DEST_RELOAD     |          \
        DMA_CONTINUOUS_ON  |                                \
        DMA_##bit##BIT_BUS | ((size)/(bit/8))))

#define H_DmaCopyArray(  dmaNo, srcp, destp, bit)           \
        H_DmaCopy(       dmaNo, srcp, destp, sizeof(srcp), bit)

#define H_DmaCopyArrayIf(dmaNo, srcp, destp, bit)           \
        H_DmaCopyIf(     dmaNo, srcp, destp, sizeof(srcp), bit)

//EHuNɓDMAŃRs[܂B
//EH_DmaCopyIf^H_DmaCopyArrayIf͏IɊ荞ݗv𔭐܂B
//EH_DmaCopyArray^H_DmaCopyArrayIf̓\[XzŜRs[܂B
//
//EF
//  dmaNo       DMAԍ
//  srcp        \[XAhX
//  destp       fXeBl[VAhX
//  size        ]oCg
//  bit         ]rbgi16|32j

//----------------------------------------------------------------------
//                  uuNcl` Rs[
//----------------------------------------------------------------------

#define V_DmaCopy(dmaNo, srcp, destp, size, bit)            \
                                                            \
    DmaSet(dmaNo, srcp, destp,  (                           \
        DMA_ENABLE         | DMA_TIMMING_V_BLANK |          \
        DMA_SRC_INC        | DMA_DEST_INC        |          \
        DMA_##bit##BIT_BUS | ((size)/(bit/8))))

#define V_DmaCopyIf(dmaNo, srcp, destp, size, bit)          \
                                                            \
    DmaSet(dmaNo, srcp, destp,  (                           \
        DMA_ENABLE         | DMA_TIMMING_V_BLANK |          \
        DMA_IF_ENABLE      |                                \
        DMA_SRC_INC        | DMA_DEST_INC        |          \
        DMA_##bit##BIT_BUS | ((size)/(bit/8))))

#define V_DmaCopyArray(  dmaNo, srcp, destp, bit)           \
        V_DmaCopy(       dmaNo, srcp, destp, sizeof(srcp), bit)

#define V_DmaCopyArrayIf(dmaNo, srcp, destp, bit)           \
        V_DmaCopyIf(     dmaNo, srcp, destp, sizeof(srcp), bit)

//EVuNɓDMAŃRs[܂B
//EV_DmaCopyIf^V_DmaCopyArrayIf͏IɊ荞ݗv𔭐܂B
//EV_DmaCopyArray^V_DmaCopyArrayIf̓\[XzŜRs[܂B
//
//EF
//  dmaNo       DMAԍ
//  srcp        \[XAhX
//  destp       fXeBl[VAhX
//  size        ]oCg
//  bit         ]rbgi16|32j

//----------------------------------------------------------------------
//                  C\cl`
//----------------------------------------------------------------------

#define DmaDispMainmem(dmaNo, srcp)                         \
                                                            \
    DmaSet(dmaNo, srcp, REG_DISP_WRAM_FIFO,  (              \
        DMA_ENABLE         | DMA_TIMMING_DISP_MMEM |        \
        DMA_SRC_INC        | DMA_DEST_FIX          |        \
		DMA_CONTINUOUS_ON  |								\
        DMA_32BIT_BUS      | (4)))

//EC̃C[W\DMA]s܂B
//
//EF
//  dmaNo       DMAԍ
//  srcp        \[XAhX

//----------------------------------------------------------------------
//                  WIgehen|cl`
//----------------------------------------------------------------------

#define GX_Dma(dmaNo, srcp, length)                         \
                                                            \
    DmaSetAsync(dmaNo, srcp, REG_GXFIFO,  (                 \
        DMA_ENABLE         | DMA_TIMMING_GXFIFO  |          \
        DMA_SRC_INC        | DMA_DEST_FIX        |          \
        DMA_32BIT_BUS      | (length)))

#define GX_DmaIf(dmaNo, srcp, length)                       \
                                                            \
    DmaSetAsync(dmaNo, srcp, REG_GXFIFO,  (                 \
        DMA_ENABLE         | DMA_TIMMING_GXFIFO  |          \
        DMA_IF_ENABLE      |                                \
        DMA_SRC_INC        | DMA_DEST_FIX        |          \
        DMA_32BIT_BUS      | (length)))

#define GX_DmaFast(dmaNo, srcp, length)                     \
                                                            \
    DmaSet(dmaNo, srcp, REG_GXFIFO,  (                      \
        DMA_ENABLE         | DMA_TIMMING_IMM     |          \
        DMA_SRC_INC        | DMA_DEST_FIX        |          \
        DMA_32BIT_BUS      | (length)))

#define GX_DmaFastIf(dmaNo, srcp, length)                   \
                                                            \
    DmaSet(dmaNo, srcp, REG_GXFIFO,  (                      \
        DMA_ENABLE         | DMA_TIMMING_IMM     |          \
        DMA_IF_ENABLE      |                                \
        DMA_SRC_INC        | DMA_DEST_FIX        |          \
        DMA_32BIT_BUS      | (length)))

#define GX_DmaArray(  dmaNo, srcp, destp, bit)              \
        GX_Dma(  dmaNo, srcp, destp, sizeof(srcp), bit)

#define GX_DmaArrayIf(dmaNo, srcp, destp, bit)              \
        GX_DmaIf(dmaNo, srcp, destp, sizeof(srcp), bit)

#define GX_DmaArrayFast(  dmaNo, srcp, destp, bit)          \
        GX_DmaFast(  dmaNo, srcp, destp, sizeof(srcp), bit)

#define GX_DmaArrayFastIf(dmaNo, srcp, destp, bit)          \
        GX_DmaFastIf(dmaNo, srcp, destp, sizeof(srcp), bit)

//EWIgFIFO̗vɂDMAŃRs[܂B
//EGX_DmaIf^GX_DmaArrayIf^GX_DmaFastIf^GX_DmaArrayFastIf
//  ͏IɊ荞ݗv𔭐܂B
//EGX_DmaArray^GX_DmaArrayIf^GX_DmaArrayFast^GX_DmaArrayFastIf
//  ̓\[XzŜRs[܂B
//
//EF
//  dmaNo       DMAԍ
//  srcp        \[XAhX
//  size        ]oCg


//----------------------------------------------------------------------
//                      cl` I҂
//----------------------------------------------------------------------

#define WaitDma(dmaNo)                                      \
{                                                           \
    vu32 *(dmaCntp) = &((vu32 *)REG_DMA0)[dmaNo * 3];       \
    while (dmaCntp[2] & DMA_ENABLE) ;                       \
}

//EDMȀI҂܂B
//
//EF
//  dmaNo       DMAԍ


//----------------------------------------------------------------------
//                      cl` Xgbv
//----------------------------------------------------------------------

#define StopDma(dmaNo)                                      \
{                                                           \
    vu16 *(dmaCntp) = &((vu16 *)REG_DMA0)[dmaNo * 6];       \
    dmaCntp[5] &= ~((DMA_TIMMING_MASK | DMA_CONTINUOUS_ON)  \
                   >> 16);                                  \
    dmaCntp[5] &= ~( DMA_ENABLE   >> 16);                   \
    {u32 dummy = dmaCntp[5];}                               \
    {u32 dummy = dmaCntp[5];}                               \
}

//EDMA~܂B
//EAADMA̎N|ꍇ͈xDMAs܂B
//
//EF
//  dmaNo       DMAԍ


//----------------------------------------------------------------------
//                      bot NA
//----------------------------------------------------------------------

#define CpuClear(data, destp, size, bit)        UTL_CpuClear##bit(data, (void *)(destp), size)

#define CpuClearArray(data, destp, bit)                     \
        CpuClear(     data, destp, sizeof(destp), bit)

//ECPURAMNAVXeR[Ăяo܂B
//ENAf[^̓X^bNɒuAfXeBl[VփRs[܂B
//ECpuClearArray̓fXeBl[VzŜNA܂B
//
//EF
//  data        NAf[^
//  destp       fXeBl[VAhX
//  size        NAoCg
//  bit         ]rbgi16|32j

//----------------------------------------------------------------------
//                      bot Rs[
//----------------------------------------------------------------------

#define CpuCopy(srcp, destp, size, bit)         UTL_CpuCopy##bit((void *)(srcp), (void *)(destp), size)

#define CpuCopyArray(srcp, destp, bit)                      \
        CpuCopy(     srcp, destp, sizeof(srcp), bit)

//ECPUŃRs[VXeR[Ăяo܂B
//ECpuCopyArray̓\[XzŜRs[܂B
//
//EF
//  srcp        \[XAhX
//  destp       fXeBl[VAhX
//  size        ]oCg
//  bit         ]rbgi16|32j

//----------------------------------------------------------------------
//                  bot NA
//----------------------------------------------------------------------

#define CpuClearFast(data, destp, size)         UTL_CpuClearFast(data, (void *)(destp), size)

#define CpuClearArrayFast(data, destp)                      \
        CpuClearFast(     data, destp, sizeof(destp))

//ECPUōRAMNAVXeR[Ăяo܂B
//ENAf[^̓X^bNɒuAfXeBl[VփRs[܂B
//ECpuClearArrayFast̓fXeBl[VzŜNA܂B
//
//EF
//  data        NAf[^
//  destp       fXeBl[VAhX
//  size        NAoCg

//----------------------------------------------------------------------
//                  bot Rs[
//----------------------------------------------------------------------

#define CpuCopyFast(srcp, destp, size)          UTL_CpuCopyFast((void *)(srcp), (void *)(destp), size)

#define CpuCopyArrayFast(srcp, destp)                       \
        CpuCopyFast(     srcp, destp, sizeof(srcp))

//ECPUōɃRs[VXeR[Ăяo܂B
//ECpuCopyArrayFast̓\[XzŜRs[܂B
//
//EF
//  srcp        \[XAhX
//  destp       fXeBl[VAhX
//  size        ]oCg


#ifdef __cplusplus
}       // extern "C"
#endif

#endif  // _IRIS_MACRO_H
